/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.dex.dexlib;

import java.util.Arrays;

import org.icedrobot.daneel.dex.Opcode;

import static org.icedrobot.daneel.dex.Opcode.*;

class OpcodesEncoder {
    // opcode categories
    private static final int INSTR = 0;
    private static final int INSTR_OP = 1;
    private static final int INSTR_UNARY_OP = 2;
    private static final int INSTR_BIN_OP = 3;
    private static final int INSTR_BIN_AND_LITERAL = 4;
    private static final int INSTR_CONST_STRING = 5;
    private static final int INSTR_CONST_U32 = 6;
    private static final int INSTR_CONST_U64 = 7;
    private static final int INSTR_CLASS = 8;
    private static final int INSTR_INSTANCEOF = 9;
    private static final int INSTR_GOTO = 10;
    private static final int INSTR_IF_TESTZ = 11;
    private static final int INSTR_IF_TEST = 12;
    private static final int INSTR_SWITCH = 13;
    private static final int INSTR_ARRAY = 14;
    private static final int INSTR_FIELD = 15;
    private static final int INSTR_METHOD = 16;
    private static final int INSTR_FILLED_NEW_ARRAY = 17;
    private static final int INSTR_NEW_ARRAY = 18;
    private static final int INSTR_FILL_ARRAY_DATA = 19;

    public static void main(String[] args) {
        Object[] assoc = {
                INSTR,
                new Opcode[] { NOP, RETURN_VOID },
                INSTR_OP,
                new Opcode[] { MOVE_RESULT, MOVE_RESULT_WIDE,
                        MOVE_RESULT_OBJECT, MOVE_EXCEPTION, RETURN,
                        MONITOR_ENTER, MONITOR_EXIT, RETURN_WIDE,
                        RETURN_OBJECT, THROW },
                INSTR_UNARY_OP,
                new Opcode[] { MOVE, MOVE_FROM16, MOVE_16, MOVE_WIDE,
                        MOVE_WIDE_FROM16, MOVE_WIDE_16, MOVE_OBJECT,
                        MOVE_OBJECT_FROM16, MOVE_OBJECT_16, NEG_INT, NOT_INT,
                        NEG_LONG, NOT_LONG, NEG_FLOAT, NEG_DOUBLE, INT_TO_LONG,
                        INT_TO_FLOAT, INT_TO_DOUBLE, LONG_TO_INT,
                        LONG_TO_FLOAT, LONG_TO_DOUBLE, FLOAT_TO_INT,
                        FLOAT_TO_LONG, DOUBLE_TO_INT, DOUBLE_TO_LONG,
                        DOUBLE_TO_FLOAT, INT_TO_BYTE, INT_TO_CHAR,
                        INT_TO_SHORT, ARRAY_LENGTH },
                INSTR_BIN_OP,
                new Opcode[] { CMPL_FLOAT, CMPG_FLOAT, CMPL_DOUBLE,
                        CMPG_DOUBLE, CMP_LONG, ADD_INT, SUB_INT, MUL_INT,
                        DIV_INT, REM_INT, AND_INT, OR_INT, XOR_INT, SHL_INT,
                        SHR_INT, USHR_INT, ADD_LONG, SUB_LONG, MUL_LONG,
                        DIV_LONG, REM_LONG, AND_LONG, OR_LONG, XOR_LONG,
                        SHL_LONG, SHR_LONG, USHR_LONG, ADD_FLOAT, SUB_FLOAT,
                        MUL_FLOAT, DIV_FLOAT, REM_FLOAT, ADD_DOUBLE,
                        SUB_DOUBLE, MUL_DOUBLE, DIV_DOUBLE, REM_DOUBLE,
                        ADD_INT_2ADDR, SUB_INT_2ADDR, MUL_INT_2ADDR,
                        DIV_INT_2ADDR, REM_INT_2ADDR, AND_INT_2ADDR,
                        OR_INT_2ADDR, XOR_INT_2ADDR, SHL_INT_2ADDR,
                        SHR_INT_2ADDR, USHR_INT_2ADDR, ADD_LONG_2ADDR,
                        SUB_LONG_2ADDR, MUL_LONG_2ADDR, DIV_LONG_2ADDR,
                        REM_LONG_2ADDR, AND_LONG_2ADDR, OR_LONG_2ADDR,
                        XOR_LONG_2ADDR, SHL_LONG_2ADDR, SHR_LONG_2ADDR,
                        USHR_LONG_2ADDR, ADD_FLOAT_2ADDR, SUB_FLOAT_2ADDR,
                        MUL_FLOAT_2ADDR, DIV_FLOAT_2ADDR, REM_FLOAT_2ADDR,
                        ADD_DOUBLE_2ADDR, SUB_DOUBLE_2ADDR, MUL_DOUBLE_2ADDR,
                        DIV_DOUBLE_2ADDR, REM_DOUBLE_2ADDR },
                INSTR_BIN_AND_LITERAL,
                new Opcode[] { ADD_INT_LIT16, RSUB_INT_LIT16, MUL_INT_LIT16,
                        DIV_INT_LIT16, REM_INT_LIT16, AND_INT_LIT16,
                        OR_INT_LIT16, XOR_INT_LIT16, ADD_INT_LIT8,
                        RSUB_INT_LIT8, MUL_INT_LIT8, DIV_INT_LIT8,
                        REM_INT_LIT8, AND_INT_LIT8, OR_INT_LIT8, XOR_INT_LIT8,
                        SHL_INT_LIT8, SHR_INT_LIT8, USHR_INT_LIT8 },
                INSTR_CONST_STRING,
                new Opcode[] { CONST_STRING, CONST_STRING_JUMBO },
                INSTR_CONST_U32,
                new Opcode[] { CONST_4, CONST_16, CONST, CONST_HIGH16 },
                INSTR_CONST_U64,
                new Opcode[] { CONST_WIDE_16, CONST_WIDE_32, CONST_WIDE,
                        CONST_WIDE_HIGH16 },
                INSTR_CLASS,
                new Opcode[] { CONST_CLASS, CHECK_CAST, NEW_INSTANCE },
                INSTR_INSTANCEOF,
                new Opcode[] { INSTANCE_OF },
                INSTR_GOTO,
                new Opcode[] { GOTO_32, GOTO, GOTO_16 },
                INSTR_IF_TESTZ,
                new Opcode[] { IF_EQZ, IF_NEZ, IF_LTZ, IF_GEZ, IF_GTZ, IF_LEZ },
                INSTR_IF_TEST,
                new Opcode[] { IF_EQ, IF_NE, IF_LT, IF_GE, IF_GT, IF_LE },
                INSTR_SWITCH,
                new Opcode[] { PACKED_SWITCH, SPARSE_SWITCH },
                INSTR_ARRAY,
                new Opcode[] { APUT, APUT_WIDE, APUT_OBJECT, APUT_BOOLEAN,
                        APUT_BYTE, APUT_CHAR, APUT_SHORT, AGET, AGET_WIDE,
                        AGET_OBJECT, AGET_BOOLEAN, AGET_BYTE, AGET_CHAR,
                        AGET_SHORT },
                INSTR_FIELD,
                new Opcode[] { IGET, IGET_WIDE, IGET_OBJECT, IGET_BOOLEAN,
                        IGET_BYTE, IGET_CHAR, IGET_SHORT, IPUT, IPUT_WIDE,
                        IPUT_OBJECT, IPUT_BOOLEAN, IPUT_BYTE, IPUT_CHAR,
                        IPUT_SHORT, SGET, SGET_WIDE, SGET_OBJECT, SGET_BOOLEAN,
                        SGET_BYTE, SGET_CHAR, SGET_SHORT, SPUT, SPUT_WIDE,
                        SPUT_OBJECT, SPUT_BOOLEAN, SPUT_BYTE, SPUT_CHAR,
                        SPUT_SHORT },
                INSTR_METHOD,
                new Opcode[] { INVOKE_VIRTUAL, INVOKE_SUPER, INVOKE_DIRECT,
                        INVOKE_STATIC, INVOKE_INTERFACE, INVOKE_VIRTUAL_RANGE,
                        INVOKE_SUPER_RANGE, INVOKE_DIRECT_RANGE,
                        INVOKE_STATIC_RANGE, INVOKE_INTERFACE_RANGE },
                INSTR_FILLED_NEW_ARRAY,
                new Opcode[] { FILLED_NEW_ARRAY, FILLED_NEW_ARRAY_RANGE },
                INSTR_NEW_ARRAY, new Opcode[] { NEW_ARRAY },
                INSTR_FILL_ARRAY_DATA, new Opcode[] { FILL_ARRAY_DATA }, };

        char[] text = new char[256];
        Arrays.fill(text, '@');
        for (int i = 0; i < assoc.length; i += 2) {
            int category = (Integer) assoc[i];
            Opcode[] opcodes = (Opcode[]) assoc[i + 1];
            for (Opcode opcode : opcodes) {
                text[opcode.ordinal()] = (char) ('A' + category);
            }
        }
        System.out.println(new String(text));
    }
}
